﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2008 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using System.Collections;
using HeuristicLab.CEDMA.DB.Interfaces;
using System.Xml;
using System.Runtime.Serialization;
using System.IO;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Data;
using HeuristicLab.DataAnalysis;
using System.Drawing;

namespace HeuristicLab.CEDMA.Core {
  public class Results {
    public static string[] CategoricalVariables {
      get { return new string[] { "TargetVariable" }; }
    }

    public static string[] OrdinalVariables {
      get { return new string[] { "ValidationMeanAbsolutePercentageError" }; }
    }

    private IStore store;
    public IStore Store {
      get { return store; }
      set {
        store = value;
      }
    }

    private Entity dataSetEntity;

    public Results(IStore store) {
      this.store = store;
    }

    internal void FilterDataSet(Entity entity) {
      this.dataSetEntity = entity;
    }

    internal IEnumerable<object[]> SelectRows(List<string> columnNames) {
      if (store == null) yield return null;

      Resource[] columnNamesRes = columnNames.Select(x => new Literal(x)).ToArray();

      var results = store.Select(new Statement(dataSetEntity, Ontology.PredicateHasModel, Ontology.AnyEntity))
         .SelectMany(x => store.Select(new SelectFilter(
           new Entity[] { (Entity)x.Property },
           new Entity[] { Ontology.PredicateModelAttribute },
           new Resource[] { Ontology.AnyEntity })))
         .SelectMany(x =>
           store.Select(
              new SelectFilter(
              new Entity[] { (Entity)x.Property },
              new Entity[] { Ontology.PredicateModelAttributeName },
              columnNamesRes)).Select(y =>
                new {
                  Model = x.Subject,
                  Attribute = (Entity)x.Property,
                  AttributeName = (Literal)y.Property
                }))
         .SelectMany(x =>
           store.Select(
             new Statement(x.Attribute, Ontology.PredicateModelAttributeValue, Ontology.AnyEntity))
             .Select(y =>
               new {
                 Model = x.Model.Uri,
                 AttributeName = x.AttributeName.Value,
                 AttributeValue = ((Literal)y.Property).Value
               })).GroupBy(x=>x.Model);


      Random random = new Random();
      foreach (var row in results) {

        yield return row.Select(x => x.AttributeValue).ToArray();
        //if (ss.Length > 0) {
        //  Record r = new Record(this, ss[0].Subject.Uri);
        //  r.Set(Record.X_JITTER, random.NextDouble() * 2.0 - 1.0);
        //  r.Set(Record.Y_JITTER, random.NextDouble() * 2.0 - 1.0);
        //  foreach (Statement s in ss) {
        //    string varName;
        //    predicateToVariableName.TryGetValue(s.Predicate, out varName);
        //    if (varName != null) {
        //      if (varName == Record.TREE_HEIGHT || varName == Record.TREE_SIZE || varName == Record.TARGET_VARIABLE) {
        //        r.Set(varName, (double)(int)((Literal)s.Property).Value);
        //      } else {
        //        r.Set(varName, (double)((Literal)s.Property).Value);
        //      }
        //    }
        //  }
        //}
      }
    }
  }
}
